﻿using System;
using DevExpress.Xpo;
using DevExpress.ExpressApp;
using DevExpress.Persistent.Base;
using DevExpress.ExpressApp.Model;
using DevExpress.Persistent.BaseImpl;
using K12MIS.Module.BusinessObjects;

namespace K12MIS.Module.BusinessObjects
{
    //Making a class abstract prevents it from creating a table in the database. Instead, any class that inherits from this class will have
    //the properties from this class included in its table.
    //This provides a very simple audit trail. For a more detailed audit trail, include the DevExpress AuditTrailModule in this project.
    [NonPersistent]

    public abstract class CustomBaseObject : XPObject
    {
        public CustomBaseObject() : base()
        {
            // This constructor is used when an object is loaded from a persistent storage.
            // Do not place any code here.
        }

        public CustomBaseObject(Session session) : base(session)
        {
            // This constructor is used when an object is loaded from a persistent storage.
            // Do not place any code here.
        }

        public override void AfterConstruction()
        {
            base.AfterConstruction();
            // Place here your initialization code.
            CreatedDate = DateTime.Now;
            CreatedBy = Session.GetObjectByKey<User>(SecuritySystem.CurrentUserId);
        }


        protected override void OnSaving()
        {
            LastModifiedDate = DateTime.Now;
            LastModifiedBy = Session.GetObjectByKey<User>(SecuritySystem.CurrentUserId);
            base.OnSaving();
        }

        protected override void OnDeleting()
        {
            DeletedDate = DateTime.Now;
            DeletedBy = Session.GetObjectByKey<User>(SecuritySystem.CurrentUserId);
            base.OnDeleting();
        }


        private DateTime _CreatedDate;
        [ModelDefault("AllowEdit", "false")]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [VisibleInDetailView(false)]
        public DateTime CreatedDate
        {
            get { return _CreatedDate; }
            set { SetPropertyValue<DateTime>("CreatedDate", ref _CreatedDate, value); }
        }


        private User _CreatedBy;
        [ModelDefault("AllowEdit", "false")]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [VisibleInDetailView(false)]
        public User CreatedBy
        {
            get { return _CreatedBy; }
            set { SetPropertyValue<User>("CreatedBy", ref _CreatedBy, value); }
        }


        private DateTime _LastModifiedDate;
        [ModelDefault("AllowEdit", "false")]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [VisibleInDetailView(false)]
        public DateTime LastModifiedDate
        {
            get { return _LastModifiedDate; }
            set { SetPropertyValue<DateTime>("LastModifiedDate", ref _LastModifiedDate, value); }
        }


        private User _LastModifiedBy;
        [ModelDefault("AllowEdit", "false")]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [VisibleInDetailView(false)]
        public User LastModifiedBy
        {
            get { return _LastModifiedBy; }
            set { SetPropertyValue<User>("LastModifiedBy", ref _LastModifiedBy, value); }
        }


        private DateTime _DeletedDate;
        [ModelDefault("AllowEdit", "false")]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [VisibleInDetailView(false)]
        public DateTime DeletedDate
        {
            get { return _DeletedDate; }
            set { SetPropertyValue<DateTime>("DeletedDate", ref _DeletedDate, value); }
        }


        private User _DeletedBy;
        [ModelDefault("AllowEdit", "false")]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [VisibleInDetailView(false)]
        public User DeletedBy
        {
            get { return _DeletedBy; }
            set { SetPropertyValue<User>("DeletedBy", ref _DeletedBy, value); }
        }
    }

}